@@ -1,5 +1,6 @@
-import time
-from django_helpers import add_app_url
+# coding=utf-8
+from django.utils.safestring import mark_safe
+from django_helpers import add_app_url, get_settings_val
 import urls
 
 from django.core.urlresolvers import reverse
@@ -7,46 +8,65 @@ from django.core.urlresolvers import reverse
 from django.db.models import Q
 from django.conf.urls import url
 from django.http import HttpResponse
-from django.utils.simplejson import dumps
+from json import dumps
 from django.template import Template, loader, RequestContext, Context
 
-from django_helpers.templatetags.static_manger import add_css_file, add_js_file, add_jquery
+from django_helpers.templatetags.static_manger import add_css_file, add_js_file, require_jquery
 
 __author__ = "ajumell"
 
 tables = dict()
 
-class DataTableColoum(object):
-    def __init__(self, field, title, searchable, sortable):
+
+def get_table(name):
+    return tables[name]()
+
+def get_value(instance, field):
+    try:
+        value = getattr(instance, field)
+    except Exception:
+        if field.find('__') > 0:
+            fields = field.split('__')
+        elif field.find('.') > 0:
+            fields = field.split('.')
+        else:
+            raise
+        value = instance
+        for field in fields:
+            value = getattr(value, field)
+    return value
+
+class DataTableColumn(object):
+    def __init__(self, field, title, searchable, sortable, title_class=None, title_width=None):
         self.field = field
         self.title = title
         self.searchable = searchable
         self.sortable = sortable
+        self.title_class = title_class
+        self.title_width = title_width
 
-    def get_html(self, request, object):
-        value = getattr(object, self.field)
-        return value
-
+    def get_html(self, request, instance):
+        return get_value(instance, self.field)
 
     def need_related(self):
         return self.field.find('__') > -1
 
 
-class DataTableClientRenderedColoum(DataTableColoum):
+class DataTableClientRenderedColumn(DataTableColumn):
     """
     This class is still under planning.
     """
     pass
 
 
-class DataTableTemplateColoum(DataTableColoum):
+class DataTableTemplateColumn(DataTableColumn):
     """
     A Data table column which will return a rendered
     a template instead of a database field.
     """
 
     def __init__(self, field, title, searchable, sortable, template,
-                 load_template=True, width=None, value_name='value'):
+                 load_template=True, width=None, value_name='value', title_class=None, title_width=None):
         """
         @param field: The name of the field to be passed to the template.
         @param title: Title of the column which will be displayed in the table header.
@@ -62,15 +82,15 @@ class DataTableTemplateColoum(DataTableColoum):
         @param value_name: name of the variable that has to be passed to the template.
         Its default value is "value"
         """
-        DataTableColoum.__init__(self, field, title, searchable, sortable)
+        DataTableColumn.__init__(self, field, title, searchable, sortable, title_class, title_width)
         if load_template:
             template = loader.get_template(template)
         self.template = template
         self.width = width
         self.value_name = value_name
 
-    def get_html(self, request, object):
-        value = getattr(object, self.field)
+    def get_html(self, request, instance):
+        value = getattr(instance, self.field)
         template = self.template
 
         if not isinstance(template, Template):
@@ -91,50 +111,93 @@ class DataTable(object):
     width = None
     class_name = ""
     name = None
-    coloums = []
+    columns = []
     table_id = None
     display_length = 10
     ajax_source = ''
     loading_message = 'Please wait while data is loading'
+
     ui_themes = False
+    bootstrap_theme = True
+
+    scroller = False
+    scroller_height = 200
 
     info_empty = None
     info_loading = None
     info_processing = None
 
     dom = '<f>t<plir>'
-    template = 'data_tables/table.html'
+    template = 'data-tables/table.html'
 
     def __init__(self):
-        add_css_file('css/table.css')
-        add_css_file('css/ui.css')
-        add_jquery()
-        add_js_file('js/jquery.dataTables.js')
+        require_jquery()
+        add_js_file('data-tables/js/jquery.dataTables.min.js')
 
-    def __unicode__(self):
+        if not get_settings_val('DATA_TABLES_INCLUDE_CSS', True) and not self.bootstrap_theme:
+            add_css_file('data-tables/css/jquery.dataTables.css')
+
+        if get_settings_val('DATA_TABLES_INCLUDE_UI_CSS', self.ui_themes) is True:
+            add_css_file('data-tables/css/demo_table_jui.css')
+
+        if self.bootstrap_theme:
+            if get_settings_val('BOOTSTRAP_INCLUDE_CSS', False):
+                add_css_file('django-helpers/bootstrap/css/bootstrap.min.css')
+
+            add_css_file('data-tables/css/DT_bootstrap.css')
+            add_js_file('data-tables/js/DT_bootstrap.js')
+
+        if self.scroller:
+            add_css_file('data-tables/css/dataTables.scroller.css')
+            add_js_file('data-tables/js/dataTables.scroller.min.js')
+
+    def render(self, kwargs=None, args=None):
         template = loader.get_template(self.template)
+
+        #        if self.bootstrap_theme:
+        #            self.dom = False
+
+        #        if self.scroller:
+        #            self.dom = 'frtiS'
+
         context = Context({
             'display_length': self.display_length,
             'dom': self.dom,
-            'ajax_source': reverse(self.table_id),
-            'columns': self.coloums,
+            'ajax_source': reverse(self.table_id, kwargs=kwargs, args=args),
+            'columns': self.columns,
             "table_id": self.table_id,
-            "ui_themes": self.ui_themes,
             'width': self.width,
             'loading_message': self.loading_message,
 
+            "ui_themes": self.ui_themes,
+            "bootstrap_theme": self.bootstrap_theme,
+
+            'scroller': self.scroller,
+            'scroller_height': self.scroller_height,
+
             'info_loading': self.info_loading,
             'info_processing': self.info_processing,
             'info_empty': self.info_empty
         })
         return template.render(context)
 
+    def safe_string(self, kwargs=None, args=None):
+        return mark_safe(self.render(kwargs, args))
+
+    def __unicode__(self):
+        return self.render()
+
+    def get_query(self, request, kwargs=None):
+        if hasattr(self, 'query'):
+            return self.query
+
     def get_data(self, request, kwargs):
         GET = request.GET
         gt = GET.get
 
-        columns = self.coloums
-        query = self.query.filter()
+        columns = self.columns
+        query = self.get_query(request, kwargs)
+
         extra_params = self.url_search_parameters
         if extra_params is not None:
             filter_dict = {}
@@ -176,7 +239,6 @@ class DataTable(object):
             column = columns[column_number]
             if column.sortable:
                 field = column.field
-                print sort_order
                 if sort_order == 'desc':
                     field = '-' + field
                 sortings.append(field)
@@ -212,7 +274,7 @@ def get_data(request, name, **kwargs):
     if not tables.has_key(name):
         return HttpResponse()
 
-    table = tables[name]
+    table = tables[name]()
 
     results = table.get_data(request, kwargs)
     return results
@@ -221,8 +283,10 @@ def get_data(request, name, **kwargs):
 def create_reg(name):
     return "(?P<%s>.*)/" % name
 
+
 url_prefix = 'data-table-'
 
+
 def register(data_table):
     add_app_url('data-table', urls)
 
@@ -233,15 +297,16 @@ def register(data_table):
         raise Exception("A Sub class of DataTable is required.")
 
     name = getattr(data_table, 'table_id', None)
-    if not name:
-        name = str(time.time())
-        name = name.replace('.', '')
+    if name is None:
+        raise Exception("Table ID is needed.")
 
     if not name.startswith(url_prefix):
         name = url_prefix + name
 
+    if name in tables:
+        raise Exception("A Data table with the given ID already exists.")
+
     setattr(data_table, 'table_id', name)
-    data_table = data_table()
     tables[name] = data_table
     reg = "%s/" % name
     extra_url_parameters = data_table.url_search_parameters
