@@ -1,4 +1,5 @@
-from django_helpers.views.helpers import render_template
+# coding=utf-8
+from django_helpers.helpers.views import render_template
 
 __author__ = 'ajumell'
 
